#!/bin/bash
# Copyright 2019-2020 Martin Engel, Vision Components GmbH
# Use at your own Risk!

function _main()
{
	_check_running_driver
	_reconfigure_gpios_by_platform
	_readout_values
}

function _reconfigure_gpios_raspi3Bplus()
{
	_check_for_package  "raspi-gpio"

	echo  ""
	echo  ""
	raspi-gpio  get

	echo  ""
	echo  ""
	echo  "Warning: GPIO pins 0, 1, 44 and 45 will change behaviour,"
	echo  "         as we assume the I2C0 is default at GPIO pins 0 and 1 (switching to input)"
	echo  "         and we change pins 44 and 45 to alt1 to become SDA0 and SCL0 for I2C0."
	echo  "         There then will be probing for an I2C client on pin 44 and 45."
	echo  "         this means if devices are connected to these pins other than the mipi module,"
	echo  "         they may be damaged! Also quit all programs using these pins beforehand."
	echo  ""
	echo  " Proceed ((y/N) followed by return)?"
	read response
	if [ "$response" != "y" ] ;then
		echo  " Abort by user request."
		echo  ""
		exit
	fi

	#reconfigure gpio pins
	raspi-gpio set  0  ip
	raspi-gpio set  1  ip
	raspi-gpio set 44  a1
	raspi-gpio set 45  a1

	echo  ""
	echo  ""
	raspi-gpio  get

	echo  ""
	echo  ""
	echo  " GPIO pins 44 and 45 must now be the only one labelled as SDA0 and SCL0."
	echo  ""
	echo  " Proceed ((y/N) followed by return)?"
	read response
	if [ "$response" != "y" ] ;then
		echo  " Abort by user request."
		echo  ""
		exit
	fi
}

function _reconfigure_gpios_raspiZero()
{
	_check_for_package  "raspi-gpio"

	echo  ""
	echo  ""
	raspi-gpio  get

	echo  ""
	echo  ""
	echo  "Warning: GPIO pins 0, 1, 28 and 29 will change behaviour,"
	echo  "         as we assume the I2C0 is default at GPIO pins 0 and 1 (switching to input)"
	echo  "         and we change pins 28 and 29 to alt1 to become SDA0 and SCL0 for I2C0."
	echo  "         There then will be probing for an I2C client on pin 28 and 29."
	echo  "         this means if devices are connected to these pins other than the mipi module,"
	echo  "         they may be damaged! Also quit all programs using these pins beforehand."
	echo  ""
	echo  " Proceed ((y/N) followed by return)?"
	read response
	if [ "$response" != "y" ] ;then
		echo  " Abort by user request."
		echo  ""
		exit
	fi

	#reconfigure gpio pins
	raspi-gpio set  0  ip
	raspi-gpio set  1  ip
	raspi-gpio set 28  a0
	raspi-gpio set 29  a0

	echo  ""
	echo  ""
	raspi-gpio  get

	echo  ""
	echo  ""
	echo  " GPIO pins 44 and 45 must now be the only one labelled as SDA0 and SCL0."
	echo  ""
	echo  " Proceed ((y/N) followed by return)?"
	read response
	if [ "$response" != "y" ] ;then
		echo  " Abort by user request."
		echo  ""
		exit
	fi
}
function _reconfigure_gpios_raspiCM3()
{
	_check_for_package  "raspi-gpio"

	echo  ""
	echo  ""
	raspi-gpio  get

	echo  ""
	echo  ""
	echo  "Warning: There will be probing for an I2C client on pins actually configured as SDA0 and SCL0."
	echo  "         this means if devices are connected to these pins other than the mipi module,"
	echo  "         they may be damaged! Also quit all programs using these pins beforehand."
	echo  ""
	echo  " Proceed ((y/N) followed by return)?"
	read response
	if [ "$response" != "y" ] ;then
		echo  " Abort by user request."
		echo  ""
		exit
	fi
}

function _readout_values()
{
	_check_for_package  "i2c-tools"

	#load i2c-dev kernel module for accessing data over I2C

	modprobe i2c-dev

	# Sequence for getting the module info:  Read 0x1000 at device 0x10

	i2cset -y 0 0x10  0x10 0x00

	#_i2cGetNextBytes 256 hstr
	#echo $hstr
	#exit

	_i2cGetNextString 12   strMagicNr
	_i2cGetNextString 32   strModManufacterer
	_i2cGetNext16BitStrNum strModManufactererId
	_i2cGetNextString 8    strSenManufacterer
	_i2cGetNextString 16   strSenType
	_i2cGetNext16BitStrNum strModId
	_i2cGetNext16BitStrNum strModRevision
	_i2cGetNext16BitStrNum strAddrChipIDHighByte
	_i2cGetNext16BitStrNum strAddrChipIDLowByte

	_i2cGetNext16BitStrNum strChipRevision
	_i2cGetNext16BitStrNum strIdle
	_i2cGetNext16BitStrNum strHorStartHighByte
	_i2cGetNext16BitStrNum strHorStartLowByte
	_i2cGetNext16BitStrNum strVertStartHighByte
	_i2cGetNext16BitStrNum strVertStartLowByte
	_i2cGetNext16BitStrNum strHorEndHighByte
	_i2cGetNext16BitStrNum strHorEndLowByte
	_i2cGetNext16BitStrNum strVertEndHighByte
	_i2cGetNext16BitStrNum strVertEndLowByte
	_i2cGetNext16BitStrNum strHorWidthHighByte
	_i2cGetNext16BitStrNum strHorWidthLowByte
	_i2cGetNext16BitStrNum strVertHeightHighByte
	_i2cGetNext16BitStrNum strVertHeightLowByte
	_i2cGetNext16BitStrNum strExposureHighByte
	_i2cGetNext16BitStrNum strExposureMidByte
	_i2cGetNext16BitStrNum strExposureLowByte
	_i2cGetNext16BitStrNum strGainHighByte
	_i2cGetNext16BitStrNum strGainLowByte

	_i2cGetNextBytes 14    strReserved0

	_i2cGetNext16BitStrNum strModeNr
	_i2cGetNext16BitStrNum strModeBytes

	echo "-------------------------------"
	echo "strMagicNr              $strMagicNr"
	echo "strModManufacterer      $strModManufacterer"
	echo "strModManufactererId    $strModManufactererId"
	echo "strSenManufacterer      $strSenManufacterer"
	echo "strSenType              $strSenType"
	echo "strModId                $strModId"
	echo "strModRevision          $strModRevision"
	echo "-------------------------------"
	echo "Sensor Register adresses follow"
	echo "-------------------------------"
	echo "strAddrChipIDHighByte   $strAddrChipIDHighByte"
	echo "strAddrChipIDLowByte    $strAddrChipIDLowByte"
	echo "strChipRevision         $strChipRevision"
	echo "strIdle                 $strIdle"
	echo "strHorStartHighByte     $strHorStartHighByte"
	echo "strHorStartLowByte      $strHorStartLowByte"
	echo "strVertStartHighByte    $strVertStartHighByte"
	echo "strVertStartLowByte     $strVertStartLowByte"
	echo "strHorEndHighByte       $strHorEndHighByte"
	echo "strHorEndLowByte        $strHorEndLowByte"
	echo "strVertEndHighByte      $strVertEndHighByte"
	echo "strVertEndLowByte       $strVertEndLowByte"
	echo "strHorWidthHighByte     $strHorWidthHighByte"
	echo "strHorWidthLowByte      $strHorWidthLowByte"
	echo "strVertHeightHighByte   $strVertHeightHighByte"
	echo "strVertHeightLowByte    $strVertHeightLowByte"
	echo "strExposureHighByte     $strExposureHighByte"
	echo "strExposureMidByte      $strExposureMidByte"
	echo "strExposureLowByte      $strExposureLowByte"
	echo "strGainHighByte         $strGainHighByte"
	echo "strGainLowByte          $strGainLowByte"
	echo "-------------------------------"
	echo "strModeNr               $strModeNr"
	echo "strModeBytes            $strModeBytes"


	_numStrToNum "$strModeNr" numModeNr
	j=0
	while [ $j -lt $numModeNr ] ; do
		_i2cGetNext32BitStrNum strMipiDataRateMbs
		_i2cGetNext8BitStrNum  strMipiDataLanes
		_i2cGetNext8BitStrNum  strMipiFormat
		_i2cGetNext8BitStrNum  strMipiType
		_i2cGetNextBytes 9     strReserved0

		_numStrToNum "$strMipiDataRateMbs" numMipiDataRateMbs
		_numStrToNum "$strMipiDataLanes"   numMipiDataLanes

		echo "-------------------------------"
		echo "strMipiDataRateMbs      $strMipiDataRateMbs ($numMipiDataRateMbs)"
		echo "strMipiDataLanes        $strMipiDataLanes ($numMipiDataLanes)"
		echo "strMipiFormat           $strMipiFormat"
		echo "strMipiType             $strMipiType"
		echo "strReserved0            $strReserved0"

		let j=j+1
	done
	echo "-------------------------------"
}


function _check_running_driver()
{
	lsmod | grep vc_mipi_ > /dev/null
	ERR=$?

	if [ $ERR -eq 0 ]; then
		echo  "Error, Running drivers vc_mipi_.. must be unloaded first!"
		echo  ""
		exit
	fi

	return 0
}

function _reconfigure_gpios_by_platform()
{
	local  _MODEL_RASPICM3PLUS="Raspberry Pi Compute Module 3 Plus"
	local  _MODEL_RASPI3BPLUS="Raspberry Pi 3 Model B Plus"
	local  _MODEL_RASPI4B="Raspberry Pi 4 Model B"
	local  _MODEL_RASPIZERO="Raspberry Pi Zero"
	local  _MODEL _RC

	read -r _MODEL  < /proc/device-tree/model
	if ! ([ -f /proc/device-tree/model ]\
	     && (\
		   [[ "$_MODEL" == "$_MODEL_RASPI3BPLUS"*  ]] \
		|| [[ "$_MODEL" == "$_MODEL_RASPI4B"*      ]] \
		|| [[ "$_MODEL" == "$_MODEL_RASPIZERO"*    ]] \
		|| [[ "$_MODEL" == "$_MODEL_RASPICM3PLUS"* ]] \
		))
	then
		echo  ""
		echo  ""
		echo  " Error identifying the platform, GPIOS may be wrong!"
		echo  " Proceed at your own Risk ((y/N) followed by return)?"
		read response
		if [ "$response" != "y" ] ;then
			echo  " Abort by user request."
			echo  ""
			exit
		fi
	else
		if [[ "$_MODEL" == "$_MODEL_RASPI3BPLUS"* ]] ; then
			_reconfigure_gpios_raspi3Bplus
		fi
		if [[ "$_MODEL" == "$_MODEL_RASPI4B"* ]] ; then
			_reconfigure_gpios_raspi3Bplus
		fi
		if [[ "$_MODEL" == "$_MODEL_RASPIZERO"* ]] ; then
			_reconfigure_gpios_raspiZero
		fi
		if [[ "$_MODEL" == "$_MODEL_RASPICM3PLUS"* ]] ; then
			_reconfigure_gpios_raspiCM3
		fi
	fi
}

#
#  @param   $1   Name of the debian package to be present
#
function _check_for_package()
{
	dpkg -l | grep $1  > /dev/null
	ERR=$?
	if [ $ERR -ne 0 ]; then
		echo  ""
		echo  ""
		echo  " Error, missing dependencies: This tool needs the \'$1\' package installed!"
		echo  " To do so, execute the following command:"
		echo  ""
		echo  "   sudo  apt install  $1"
		echo  ""
		exit
	fi
}






#
#  @param   $1   number of Bytes to get
#  @param   $2   Name for the output string variable, formatted as hex sequence, e.g. "\x65\x66"
#
_i2cGetNextBytes()
{
	local ERR=0
	local i=0
	local str=""

	while [ $i -lt $1 ] ; do
		local val=$(i2cget -y 0 0x10)
		ERR=$?
		if [ $ERR -ne 0 ]; then
			return $ERR
		fi

		str+="$( printf "%s" "\\"${val:1} )"

		let i=i+1
	done

	eval $2="'$str'"
	ERR=$?

	return $ERR
}

#
#  @param   $1   Name for the output string variable, formatted as hex sequence, e.g. "0xaabb" where byte order swapped
#
_i2cGetNext8BitStrNum()
{
	local ERR=0
	local str=""

	local number=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	eval $1="'0x${number:2}'"
	ERR=$?

	return $ERR
}

#
#  @param   $1   Name for the output string variable, formatted as hex sequence, e.g. "0xaabb" where byte order swapped
#
_i2cGetNext16BitStrNum()
{
	local ERR=0
	local str=""

	local lower=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	local upper=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	eval $1="'0x${upper:2}${lower:2}'"
	ERR=$?

	return $ERR
}

#
#  @param   $1   Name for the output string variable, formatted as hex sequence, e.g. "0xaabbccdd" where byte order swapped
#
_i2cGetNext32BitStrNum()
{
	local ERR=0
	local str=""

	local lowest=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	local lowmid=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	local uppmid=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	local uppest=$(i2cget -y 0 0x10)
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	eval $1="'0x${uppest:2}${uppmid:2}${lowmid:2}${lowest:2}'"
	ERR=$?

	return $ERR
}

#
#  @param   $1   Name for the string variable, formatted as hex sequence, e.g. "\x65\x66"
#  @param   $2   Name for the output string variable, stopping at '\x00'
#
_hexStrToStr()
{
	eval $2="'$(printf ${1%%"\x00"*} )'"
	ERR=$?

	return $ERR
}


#
#  @param   $1   Name for the string variable, formatted as hex sequence, e.g. "0x6566"
#  @param   $2   Name for the output number
#
_numStrToNum()
{
	eval $2=$(( $1 ))
	ERR=$?

	return $ERR
}

#
#  @param   $1   number of Bytes to get
#  @param   $2   Name for the output string variable, stopping at '\x00'
#
_i2cGetNextString()
{
	local ERR=0
	local hstr=""

	_i2cGetNextBytes $1 hstr
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	_hexStrToStr "$hstr" $2
	ERR=$?

	return $ERR
}


#
#  @param   $1   Name for the output number variable
#
_i2cGetNext16BitNumber()
{
	local ERR=0
	local strNum=""

	_i2cGetNext16BitStrNum strNum
	ERR=$?
	if [ $ERR -ne 0 ]; then
		return $ERR
	fi

	_numStrToNum "$strNum" $1
	ERR=$?

	return $ERR
}




_main "$@"




exit

